Index: src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.cpp
===================================================================
--- src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.cpp	(rvision 11936)
+++ src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.cpp	(copie de travail)
@@ -32,6 +32,9 @@
 #include "stdafx.h"
 #include "theoraencodeinputpin.h"
 
+/* Define to adapt theora frame rate to DirectShow clock */
+//#define ADAPT_FRAME_RATE 1
+
 TheoraEncodeInputPin::TheoraEncodeInputPin(AbstractTransformFilter* inParentFilter, CCritSec* inFilterLock, AbstractTransformOutputPin* inOutputPin, vector<CMediaType*> inAcceptableMediaTypes)
 	:	AbstractTransformInputPin(inParentFilter, inFilterLock, inOutputPin, NAME("TheoraEncodeInputPin"), L"YV12 In", inAcceptableMediaTypes)
 	,	mXOffset(0)
@@ -43,6 +46,8 @@
 	,	mVideoFormat(NULL)
 	
 
+	, numFrames(0)
+	, timeStart(0), timeEnd(0)
 {
 	//debugLog.open("g:\\logs\\theoencfiltinput.log", ios_base::out);
 	mYUV.y = NULL;
@@ -1168,6 +1173,11 @@
 	return 0;
 }
 
+void TheoraEncodeInputPin::SetInputSampleTime(REFERENCE_TIME pTimeStart, REFERENCE_TIME pTimeEnd)
+{
+	timeStart = pTimeStart/10000;
+	timeEnd = pTimeEnd/10000;
+}
 
 
 //PURE VIRTUALS
@@ -1237,7 +1247,17 @@
 		throw 0;
 	}
 	
+#ifdef ADAPT_FRAME_RATE
+	__int64 frameTime = (1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+	__int64 curTheoraTime = (numFrames * 1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
 
+	// Skip a frame if we are too late
+	if (((signed) (curTheoraTime-timeStart)) > frameTime) {
+		//debugLog<<" too late: "<< curTheoraTime << "," << timeStart << ":" << frameTime <<endl;
+		return S_OK;
+	}
+#endif
+
 	StampedOggPacket* locPacket = mTheoraEncoder.encodeTheora(&mYUV);
 	if (locPacket == NULL) {
 		//debugLog<<"Encode returns NULL"<<endl;
@@ -1250,9 +1270,51 @@
 	//We still own the packet after this, we have to delete it.
 	locHR = deliverData(locFrameStart, locFrameEnd, locPacket->packetData(), locPacket->packetSize());
 	delete locPacket;
+
+#ifdef ADAPT_FRAME_RATE
+	if (!FAILED(locHR)) {
+		++numFrames;
+		locHR = encodeMoreFrames();
+	}
+#endif
+
 	return locHR;
+}
 
+HRESULT TheoraEncodeInputPin::encodeMoreFrames(void)
+{
+	HRESULT locHR = S_OK;
+
+	__int64 frameTime = (1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+	__int64 curTheoraTime = (numFrames * 1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+
+	/* Resend previous packet, if we are too fast */
+	while (((signed) (timeStart-curTheoraTime)) > frameTime) {
+		LONGLONG locFrameStart = mUptoFrame;
+		LONGLONG locFrameEnd = 0;
+
+		StampedOggPacket* locPacket = mTheoraEncoder.encodeTheora(&mYUV);
+		if (locPacket == NULL) {
+			//debugLog<<"Encode returns NULL"<<endl;
+			return S_FALSE;
+		}
+		locFrameEnd		= mUptoFrame = locPacket->endTime();
+		//debugLog<<"Delivering..."<<endl;
+
+		//We still own the packet after this, we have to delete it.
+		locHR = deliverData(locFrameStart, locFrameEnd, locPacket->packetData(), locPacket->packetSize());
+		delete locPacket;
+		if (FAILED(locHR)) {
+			break;
+		}
+
+		++numFrames;
+		curTheoraTime = (numFrames * 1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+	}
+
+	return locHR;
 }
+
 bool TheoraEncodeInputPin::ConstructCodec() {
 
 	//debugLog<<"Contructing codec..."<<endl;
Index: src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.h
===================================================================
--- src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.h	(rvision 11936)
+++ src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.h	(copie de travail)
@@ -111,5 +111,10 @@
 	//fstream debugLog;
 	//
 
-	
+
+	/* To adapt theora frame rate to variable directshow clock */
+	unsigned __int64 timeStart, timeEnd;
+	unsigned __int64 numFrames;
+	HRESULT encodeMoreFrames(void);
+	void SetInputSampleTime(REFERENCE_TIME pTimeStart, REFERENCE_TIME pTimeEnd);	
 };